# Generated by Django 4.2.24 on 2025-10-01 17:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("lti_tool", "0006_ltiregistration_audience"),
    ]

    operations = [
        migrations.AddField(
            model_name="lticontext",
            name="lti1p1_id_on_platform",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="LTI 1.1 ID"
            ),
        ),
        migrations.AddField(
            model_name="ltiplatforminstance",
            name="lti1p1_id_on_platform",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="LTI 1.1 ID"
            ),
        ),
        migrations.AddField(
            model_name="ltiresourcelink",
            name="lti1p1_id_on_platform",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="LTI 1.1 ID"
            ),
        ),
        migrations.AddField(
            model_name="ltiuser",
            name="lti1p1_id_on_platform",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="LTI 1.1 ID"
            ),
        ),
    ]
